#define BOTTOM_ROW       LV_SYMBOL_NEXT, "CTRL", " ", LV_SYMBOL_LEFT, LV_SYMBOL_UP, LV_SYMBOL_DOWN, LV_SYMBOL_RIGHT, ""
#define CTRL_BOTTOM_ROW  2, 2, 6, 2, 2, 2, 2

/* Keyboard map:
 *  1st line: 11 keys
 *  2nd line: 11 keys
 *  3rd line: 12 keys
 *  4th line: 7 keys
 */

static void lv_terminal_keyboard_configure(lv_obj_t * kb);

static const char * const kb_map_lc[] = {"1#", "q", "w", "e", "r", "t", "y", "u", "i", "o", "p", LV_SYMBOL_BACKSPACE, "\n",
                                                 "ABC", "a", "s", "d", "f", "g", "h", "j", "k", "l", LV_SYMBOL_NEW_LINE, "\n",
                                                 "_", "-", "z", "x", "c", "v", "b", "n", "m", ".", ",", ":", "\n",
                                                 BOTTOM_ROW
                                                };

static const lv_btnmatrix_ctrl_t kb_ctrl_lc_map[] = {
    LV_KEYBOARD_CTRL_BTN_FLAGS | 5, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 7,
    LV_KEYBOARD_CTRL_BTN_FLAGS | 6, 3, 3, 3, 3, 3, 3, 3, 3, 3, 7,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    CTRL_BOTTOM_ROW
};

static const char * const kb_map_uc[] = {"1#", "Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", LV_SYMBOL_BACKSPACE, "\n",
                                                 "abc", "A", "S", "D", "F", "G", "H", "J", "K", "L", LV_SYMBOL_NEW_LINE, "\n",
                                                 "_", "-", "Z", "X", "C", "V", "B", "N", "M", ".", ",", ":", "\n",
                                                 BOTTOM_ROW
                                                };

static const lv_btnmatrix_ctrl_t kb_ctrl_uc_map[] = {
    LV_KEYBOARD_CTRL_BTN_FLAGS | 5, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 7,
    LV_KEYBOARD_CTRL_BTN_FLAGS | 6, 3, 3, 3, 3, 3, 3, 3, 3, 3, 7,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    CTRL_BOTTOM_ROW
};

static const char * const kb_map_spec[] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", LV_SYMBOL_BACKSPACE, "\n",
                                                   "abc", "+", "-", "/", "*", "=", "%", "!", "?", "#", "<", ">", "\n",
                                                   "\\",  "@", "$", "(", ")", "{", "}", "[", "]", ";", "\"", "'", "\n",
                                                   BOTTOM_ROW
                                                  };

static const lv_btnmatrix_ctrl_t kb_ctrl_spec_map[] = {
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 2,
    LV_KEYBOARD_CTRL_BTN_FLAGS | 2, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
    CTRL_BOTTOM_ROW
};

static void lv_terminal_keyboard_configure(lv_obj_t * kb)
{
    lv_keyboard_set_map(kb, LV_KEYBOARD_MODE_TEXT_LOWER, kb_map_lc);
    lv_keyboard_set_ctrl_map(kb, LV_KEYBOARD_MODE_TEXT_LOWER, kb_ctrl_lc_map);
    
    lv_keyboard_set_map(kb, LV_KEYBOARD_MODE_TEXT_UPPER, kb_map_uc);
    lv_keyboard_set_ctrl_map(kb, LV_KEYBOARD_MODE_TEXT_UPPER, kb_ctrl_uc_map);
    
    lv_keyboard_set_map(kb, LV_KEYBOARD_MODE_SPECIAL, kb_map_spec);
    lv_keyboard_set_ctrl_map(kb, LV_KEYBOARD_MODE_SPECIAL, kb_ctrl_spec_map);
    
    lv_keyboard_set_map(kb, LV_KEYBOARD_MODE_TEXT_UPPER, kb_map_uc);
    lv_keyboard_set_ctrl_map(kb, LV_KEYBOARD_MODE_TEXT_UPPER, kb_ctrl_uc_map);
}
